#ifndef __STL__FORMATLIST
#define __STL__FORMATLIST
#  include <iostream>
   namespace stl
   {
      // formatlist_t:
      // helper class that does the main work for list formatting

      template< typename ForwardIterator >
      class formatlist_t
      {
         private: // list format control
            char                       __openlist;
            char                       __closelist;
            char                       __separator;
         public:
            inline char                          openlist()  const throw();
            inline char                          closelist() const throw();
            inline char                          separator() const throw();
         public:
            inline formatlist_t &                format( char, char, char ) throw(); // open, close, separator
            inline formatlist_t &                format( char, char )       throw(); // open, close
            inline formatlist_t &                format( char )             throw(); // separator
         private: // spacing control
            bool                       __spaceBeforeSep;
            bool                       __spaceAfterSep;
            bool                       __spaceAroundList;
         public:
            inline bool                          spaceBeforeSep()  const throw();
            inline bool                          spaceAfterSep()   const throw();
            inline bool                          spaceAroundList() const throw();
         public:
            inline formatlist_t &                space( bool, bool, bool ) throw(); // before, after, around
            inline formatlist_t &                space( bool, bool )       throw(); // before, after
            inline formatlist_t &                space( bool )             throw(); // around
         private: // list range
            ForwardIterator            first;
            ForwardIterator            last;
         public:
            inline ForwardIterator               begin() const throw();
            inline ForwardIterator               end()   const throw();
         public:
            typedef ForwardIterator                        iterator;
         public:
            inline           formatlist_t( ForwardIterator, ForwardIterator );
      };

      template< typename ForwardIterator >
      char formatlist_t< ForwardIterator >::openlist() const throw()
      {
         return( __openlist );
      }
      template< typename ForwardIterator >
      char formatlist_t< ForwardIterator >::closelist() const throw()
      {
         return( __closelist );
      }
      template< typename ForwardIterator >
      char formatlist_t< ForwardIterator >::separator() const throw()
      {
         return( __separator );
      }

      template< typename ForwardIterator >
      formatlist_t< ForwardIterator > & formatlist_t< ForwardIterator >::format( char ol, char cl, char s ) throw()
      {
         __openlist  = ol;
         __closelist = cl;
         __separator = s;
         return( *this );
      }
      template< typename ForwardIterator >
      formatlist_t< ForwardIterator > & formatlist_t< ForwardIterator >::format( char ol, char cl ) throw()
      {
         __openlist  = ol;
         __closelist = cl;
         return( *this );
      }
      template< typename ForwardIterator >
      formatlist_t< ForwardIterator > & formatlist_t< ForwardIterator >::format( char s ) throw()
      {
         __separator = s;
         return( *this );
      }

      template< typename ForwardIterator >
      bool formatlist_t< ForwardIterator >::spaceBeforeSep() const throw()
      {
         return( __spaceBeforeSep );
      }
      template< typename ForwardIterator >
      bool formatlist_t< ForwardIterator >::spaceAfterSep() const throw()
      {
         return( __spaceAfterSep );
      }
      template< typename ForwardIterator >
      bool formatlist_t< ForwardIterator >::spaceAroundList() const throw()
      {
         return( __spaceAroundList );
      }

      template< typename ForwardIterator >
      formatlist_t< ForwardIterator > &
      formatlist_t< ForwardIterator >::space( bool bs, bool as, bool al ) throw()
      {
         __spaceBeforeSep  = bs;
         __spaceAfterSep   = as;
         __spaceAroundList = al;
         return( *this );
      }
      template< typename ForwardIterator >
      formatlist_t< ForwardIterator > &
      formatlist_t< ForwardIterator >::space( bool bs, bool as ) throw()
      {
         __spaceBeforeSep  = bs;
         __spaceAfterSep   = as;
         return( *this );
      }
      template< typename ForwardIterator >
      formatlist_t< ForwardIterator > &
      formatlist_t< ForwardIterator >::space( bool al ) throw()
      {
         __spaceAroundList = al;
         return( *this );
      }

      template< typename ForwardIterator >
      ForwardIterator formatlist_t< ForwardIterator >::begin() const throw()
      {
         return( first );
      }
      template< typename ForwardIterator >
      ForwardIterator formatlist_t< ForwardIterator >::end() const throw()
      {
         return( last );
      }

      template< typename ForwardIterator >
      formatlist_t< ForwardIterator >::formatlist_t( ForwardIterator f, ForwardIterator l ):
         first( f ),
         last(  l )
      {
         __openlist  = '[';
         __closelist = ']';
         __separator = ',';
         __spaceBeforeSep  = false;
         __spaceAfterSep   = true;
         __spaceAroundList = true;
      }

      // output routine for list formatting

      template< typename CharT, class TraitsT, typename ForwardIterator >
      inline std::basic_ostream< CharT, TraitsT > & operator<<
      (
         std::basic_ostream< CharT, TraitsT >  & os,
         formatlist_t< ForwardIterator > const & fi
      )
      {
         ForwardIterator               first = fi.begin();
         ForwardIterator               last  = fi.end();

         if( fi.openlist() != static_cast< CharT >( '\0' ))
            os << static_cast< CharT >( fi.openlist());
         if( fi.spaceAroundList())               os << static_cast< CharT >( ' ' );

         while( first != last )
         {
            os << *first++;
            if( first != last )
            {
               if( fi.spaceBeforeSep())          os << static_cast< CharT >( ' ' );
               if( fi.separator() != static_cast< CharT >( '\0' ))
                  os << static_cast< CharT >( fi.separator());
               if( fi.spaceAfterSep())           os << static_cast< CharT >( ' ' );
            }
         }

         if( fi.spaceAroundList())               os << static_cast< CharT >( ' ' );
         if( fi.closelist() != static_cast< CharT >( '\0' ))
            os << static_cast< CharT >( fi.closelist());
         return( os );
      }

      // formatlist manipulator: these are the main forms of the manipulator function

      template< typename ForwardIterator >
      inline formatlist_t< ForwardIterator > formatlist( ForwardIterator first, ForwardIterator last )
      {
         return( formatlist_t< ForwardIterator >( first, last ));
      }
      template< typename Container >
      inline formatlist_t< typename Container::iterator > formatlist( Container & c )
      {
         return( formatlist_t< typename Container::iterator >( c.begin(), c.end()));
      }
   }
#endif
